/***************************************************************************
 *   Copyright (C) 2007 by Andrea Moro   *
 *   andrea_moro@email.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/
#include <QtGui>
#include "ADFGVX.h"
#include "mainWindow.h"


ADFGVX::ADFGVX(mainWindow *main,QString text, QString title)
{
	setupUi(this);
	workText=text;
	workTitle=title;
	mw=main;
	encryptButton->setEnabled(false);
	decryptButton->setEnabled(false);
	QRegExp re("[a-zA-Z]*");
	QRegExpValidator *v = new QRegExpValidator(re,this);
	int i,j,posx,posy=96;
	for(i=0;i<6;i++)
	{
		posx=62;
		for(j=0;j<6;j++)
		{
			cell[i][j]= new QLineEdit(this);
			cell[i][j]->setMaxLength(1);
			cell[i][j]->setGeometry(posx,posy,20,25);
			cell[i][j]->setVisible(true);
			cell[i][j]->setValidator(v);
			posx+=26;
		}
		posy+=31;
	}
	stdMatr();
	createConnections();
}
void ADFGVX::stdMatr()
{
	int i,j,k=0;
	for(i=0;i<6;i++)
		for(j=0;j<6;j++)
		{
			if('A'+(i*6)+j<='Z')
				cell[i][j]->setText(QChar('A'+(i*6)+j));
			else
				cell[i][j]->setText(QChar('0'+k++));
		}
}
void ADFGVX::eraseMatr()
{
	int i,j;
	for(i=0;i<6;i++)
		for(j=0;j<6;j++)
			cell[i][j]->setText(QChar('*'));
}
void ADFGVX::rndMatr()
{
	int idx[36],i,j,mod=35,tmp,old;
	for(i=0;i<36;i++)
		idx[i]=i;
	for(i=0;i<36;i++)
	{
		tmp=(int)(((double)qrand()/RAND_MAX)*mod);
		old=idx[mod];
		idx[mod]=idx[tmp];
		idx[tmp]=old;
		mod--;
	}
	for(i=0;i<6;i++)
		for(j=0;j<6;j++)
			if(idx[(i*6)+j]<26)
				cell[i][j]->setText(QChar('A'+idx[(i*6)+j]));
			else
				cell[i][j]->setText(QChar('0'+idx[(i*6)+j]-26));
}
void ADFGVX::strMatr()
{
	QString text = QInputDialog::getText(this,"CrypTooLinux",
		tr("The string must contain every 36 characters "
		"and numbers(A-Z,0-9) exactly once:"));
	if(text.length()==0)
		return;
	if(text.length()!=36)
	{
		QMessageBox::warning(this, tr("Warning"),
			tr("There isn't all the characters and numbers(A-Z,0-9)"));
		return;
	}
	text=text.toUpper();
	int idx[36],i;
	for(i=0;i<36;i++)
		idx[i]=0;
	for(i=0;i<36;i++)
	{
		if((text[i]>=QChar('A'))&&(text[i]<=QChar('Z')))
			idx[text[i].toAscii()-'A']=1;
		else if((text[i]>=QChar('0'))&&(text[i]<=QChar('9')))
			idx[text[i].toAscii()-'0'+26]=1;
		else
		{
			QMessageBox::warning(this, tr("Warning"),
				tr("There is a character that is not in [A-Z0-9]"));
			return;
		}
	}
	for(i=0;i<36;i++)
		if(idx[i]==0)
		{
			QMessageBox::warning(this, tr("Warning"),
				tr("There isn't all the characters and numbers(A-Z,0-9)"));
			return;
		}
	int j;
	for(i=0;i<6;i++)
		for(j=0;j<6;j++)
			cell[i][j]->setText(QString(text[(i*6)+j]));
}
void ADFGVX::upMatr()
{
	QLineEdit *tmp=qobject_cast<QLineEdit*>(sender());
	if(tmp->text().length()!=0)
	{
		int i,j;
		bool present=false;
		for(i=0;i<6;i++)
		{
			for(j=0;j<6;j++)
				if((tmp!=cell[i][j]) && (QString::compare(cell[i][j]->text(),
					tmp->text(),Qt::CaseInsensitive)==0))
				{
				present=true;
				break;
				}
			if(present)
				break;
		}
		if(present)
		{
			QString str = tr("The character '")
				.arg(tmp->text())
				.arg(tr("' was already assigned.\nDo you want to retain the "
				"new entry?\n\nYes: The old entry will be deleted.\n\n"
				"No: Undo the Changes\n"));
			int ret = QMessageBox::warning(this, "CrypTooLinux",
				str,QMessageBox::Yes | QMessageBox::No);
			if(ret==QMessageBox::Yes)
				cell[i][j]->setText(QChar('*'));
			else if(tmp->isUndoAvailable())
				tmp->undo();
		}
		else
			tmp->setText(tmp->text().toUpper());
	}
	else
		tmp->setText(QChar('*'));
	focusNextChild();
}
void ADFGVX::upButton()
{
	pwdLine->setText(pwdLine->text().toUpper());
	if(workText.length()>0)
	{
		encryptButton->setEnabled(true);
		bool dec=true;
		int i,len=0;
		for(i=0;i<workText.length()&&dec;i++)
			if((QString::compare(QString(workText[i]),"A",Qt::CaseInsensitive)==0)||
			(QString::compare(QString(workText[i]),"D",Qt::CaseInsensitive)==0)||
			(QString::compare(QString(workText[i]),"F",Qt::CaseInsensitive)==0)||
			(QString::compare(QString(workText[i]),"G",Qt::CaseInsensitive)==0)||
			(QString::compare(QString(workText[i]),"V",Qt::CaseInsensitive)==0)||
			(QString::compare(QString(workText[i]),"X",Qt::CaseInsensitive)==0))
				len++;
			else if((QString::compare(QString(workText[i])," ",Qt::CaseInsensitive)!=0)&&
			(QString::compare(QString(workText[i]),"\n",Qt::CaseInsensitive)!=0)&&
			(QString::compare(QString(workText[i]),"\t",Qt::CaseInsensitive)!=0))
				dec=false;
		if((dec)&&(len%2==0))
			decryptButton->setEnabled(true);
		else
			decryptButton->setEnabled(false);
		QString ordered=pwdLine->text(),tmp;
		int j,max;
		for(i=0;i<ordered.length();i++)
		{
			max=i;
			for(j=i+1;j<ordered.length();j++)
				if(ordered[max].toAscii()<ordered[j].toAscii())
				max=j;
			ordered.prepend(ordered[max]);
			ordered.remove(max+1,1);
		}
		int *used = new int[pwdLine->text().length()];
		for(i=0;i<pwdLine->text().length();i++)
			used[i]=0;
		for(i=0;i<pwdLine->text().length();i++)
		{
			tmp.append("-");
			j=ordered.indexOf(pwdLine->text()[i]);
			while(used[j]==1) j++;
			used[j]=1;
			char str[10];
			sprintf(str,"%d",j+1);
			tmp.append(str);
		}
		tmp.append("-");
		transpositionLine->setText(tmp);
		delete used;
	}
	else
	{
		transpositionLine->setText("");
		encryptButton->setEnabled(false);
		decryptButton->setEnabled(false);
	}
}
void ADFGVX::upCheck()
{
	if(finalBlanksCheck->isChecked())
	{
		finalBlanks->setEnabled(true);
		finalLineCheck->setEnabled(true);
	}
	else
	{
		finalBlanks->setEnabled(false);
		finalLineCheck->setEnabled(false);
	}
	if(interBlanksCheck->isChecked())
	{
		interBlanks->setEnabled(true);
		interLineCheck->setEnabled(true);
	}
	else
	{
		interBlanks->setEnabled(false);
		interLineCheck->setEnabled(false);
	}
	if(intermediateCheck->isChecked())
	{
		interBlanksCheck->setEnabled(true);
	}
	else
	{
		interBlanksCheck->setEnabled(false);
		interBlanks->setEnabled(false);
		interLineCheck->setEnabled(false);
	}
}
void ADFGVX::createConnections()
{
	int i,j;
	for(i=0;i<6;i++)
		for(j=0;j<6;j++)
			connect(cell[i][j],SIGNAL(textEdited(const QString&)),this,SLOT(upMatr()));

	connect(intermediateCheck,SIGNAL(clicked()),this,SLOT(upCheck()));
	connect(finalBlanksCheck,SIGNAL(clicked()),this,SLOT(upCheck()));
	connect(interBlanksCheck,SIGNAL(clicked()),this,SLOT(upCheck()));
	connect(pwdLine,SIGNAL(textEdited(const QString &)),this,SLOT(upButton()));
	connect(rndButton,SIGNAL(clicked()),this,SLOT(rndMatr()));
	connect(strButton,SIGNAL(clicked()),this,SLOT(strMatr()));
	connect(eraseButton,SIGNAL(clicked()),this,SLOT(eraseMatr()));
	connect(stdButton,SIGNAL(clicked()),this,SLOT(stdMatr()));
	connect(encryptButton,SIGNAL(clicked()),this,SLOT(encrypt()));
	connect(decryptButton,SIGNAL(clicked()),this,SLOT(decrypt()));
}
void ADFGVX::decrypt()
{
	int i,j,z;
	QString tmp;
	bool present;
	for(i=0;i<pwdLine->text().length();i++)
	{
		present=false;
		for(j=0;j<tmp.length();j++)
			if(tmp[j]==pwdLine->text()[i])
			{
				present=true;
				break;
			}
		if(!present)
			tmp.append(pwdLine->text()[i]);
	}
	if(tmp.length()!=pwdLine->text().length())
	{
		QMessageBox::warning(this, "CrypTooLinux",
			tr("The sequence of the password's characters is not clearly defined.\n"
				"The redundant characters in the password have been removed!"));
		pwdLine->setText(tmp);
		upButton();
		return;
	}
	if(((interBlanksCheck->isChecked())&&(interBlanks->text().length()==0))||
		((finalBlanksCheck->isChecked())&&(finalBlanks->text().length()==0)))
	{
		QMessageBox::warning(this, "CrypTooLinux",
			tr("The number of blanks is undefined. Please define it!"));
		return;
	}
	QString title;
	if(intermediateCheck->isChecked())
	{
		title.append(tr("ADFGVX stage 1 decryption of <"));
		title.append(workTitle);
		title.append(tr(">, key <"));
		title.append(pwdLine->text());
		title.append(", ");
		for(i=0;i<6;i++)
			for(j=0;j<6;j++)
				title.append(cell[i][j]->text());
		title.append(">");
	}
	QString intermediate, ordered=pwdLine->text();
	int max;
	for(i=0;i<ordered.length();i++)
	{
		max=i;
		for(j=i+1;j<ordered.length();j++)
			if(ordered[max].toAscii()<ordered[j].toAscii())
				max=j;
		ordered.prepend(ordered[max]);
		ordered.remove(max+1,1);
	}
	int remainder,quotient,k;
	int *transposition = new int[ordered.length()];
	for(i=0;i<ordered.length();i++)
		transposition[i]=pwdLine->text().indexOf(ordered[i]);
	quotient=workText.length()/pwdLine->text().length();
	remainder=workText.length()%pwdLine->text().length();
	k=0;
	i=0;
	j=transposition[k];
	for(z=0;z<workText.length();z++)
	{
		intermediate.replace((i*pwdLine->text().length())+j,1,workText[z]);
		i++;
		if(i>=quotient+(j<remainder?1:0))
		{
			i=0;
			k=(k+1)%pwdLine->text().length();
			j=transposition[k];
		}
	}
	if(intermediateCheck->isChecked())
	{
		if(interBlanksCheck->isChecked())
		{
			z=intermediate.length()/interBlanks->text().toInt();
			for(i=1;i<=z;i++)
			{
				if(!interLineCheck->isChecked())
				intermediate.insert(
					(i*interBlanks->text().toInt())+i-1,QChar(' '));
				else
				intermediate.insert(
					(i*interBlanks->text().toInt())+i-1,QChar('\n'));
			}
		}
		mw->newSubWindow(title, intermediate);
		if(interBlanksCheck->isChecked())
		{
			z=intermediate.length()/interBlanks->text().toInt();
			for(i=z;i>0;i--)
				intermediate.remove((i*(interBlanks->text().toInt()))+i-1,1);
		}
	}
	title.clear();
	title.append(tr("ADFGVX decryption of <"));
	title.append(workTitle);
	title.append(tr(">, key <"));
	title.append(pwdLine->text());
	title.append(", ");
	for(i=0;i<6;i++)
		for(j=0;j<6;j++)
			title.append(cell[i][j]->text());
	title.append(">");
	workText.clear();
	for(i=0;i<intermediate.length();i+=2)
	{
		char str[6]={'A','D','F','G','V','X'};
		present=false;
		for(j=0;j<6;j++)
			if(str[j]==intermediate[i])
				break;
		for(z=0;z<6;z++)
			if(str[z]==intermediate[i+1])
				break;
		workText.append(cell[j][z]->text().toLower());
	}
	if(finalBlanksCheck->isChecked())
	{
		z=workText.length()/finalBlanks->text().toInt();
		for(i=1;i<=z;i++)
		{
			if(!finalLineCheck->isChecked())
				workText.insert((i*(finalBlanks->text().toInt()))+i-1,QChar(' '));
			else
				workText.insert(i*(finalBlanks->text().toInt())+i-1,QChar('\n'));
		}
	}
	mw->newSubWindow(title,workText);
	close();
}
void ADFGVX::encrypt()
{
	int i,j,z;
	QString tmp;
	bool present;
	for(i=0;i<pwdLine->text().length();i++)
	{
		present=false;
		for(j=0;j<tmp.length();j++)
			if(tmp[j]==pwdLine->text()[i])
			{
				present=true;
				break;
			}
		if(!present)
			tmp.append(pwdLine->text()[i]);
	}
	if(tmp.length()!=pwdLine->text().length())
	{
		QMessageBox::warning(this, "CrypTooLinux",
			tr("The sequence of the password's characters is not clearly defined.\n"
				"The redundant characters in the password have been removed!"));
		pwdLine->setText(tmp);
		upButton();
		return;
	}
	QString title;
	if(intermediateCheck->isChecked())
	{
		title.append(tr("ADFGVX stage 1 encryption of <"));
		title.append(workTitle);
		title.append(tr(">, key <"));
		title.append(pwdLine->text());
		title.append(", ");
		for(i=0;i<6;i++)
			for(j=0;j<6;j++)
				title.append(cell[i][j]->text());
		title.append(">");
	}
	QString intermediate;
	for(i=0;i<workText.length();i++)
		if(isalnum(workText[i].toAscii()))
		{
			char str[6]={'A','D','F','G','V','X'};
			present=false;
			for(j=0;j<6;j++)
			{
				for(z=0;z<6;z++)
					if(QString::compare(QString(workText[i]),
						cell[j][z]->text(),Qt::CaseInsensitive)==0)
					{
						present=true;
						break;
					}
				if(present)
					break;
			}
			intermediate.append(str[j]);
			intermediate.append(str[z]);
		}
	if(intermediateCheck->isChecked())
	{
		if(interBlanksCheck->isChecked())
		{
			z=intermediate.length()/interBlanks->text().toInt();
			for(i=1;i<=z;i++)
			{
				if(!interLineCheck->isChecked())
				intermediate.insert((i*(interBlanks->text().toInt()))+i-1,QChar(' '));
				else
				intermediate.insert(i*(interBlanks->text().toInt())+i-1,QChar('\n'));
			}
		}
		mw->newSubWindow(title,intermediate);
		if(interBlanksCheck->isChecked())
		{
			z=intermediate.length()/interBlanks->text().toInt();
			for(i=z;i>0;i--)
				intermediate.remove((i*(interBlanks->text().toInt()))+i-1,1);
		}
	}
	title.clear();
	title.append(tr("ADFGVX encryption of <"));
	title.append(workTitle);
	title.append(tr(">, key <"));
	title.append(pwdLine->text());
	title.append(", ");
	for(i=0;i<6;i++)
		for(j=0;j<6;j++)
			title.append(cell[i][j]->text());
	title.append(">");
	QString ordered=pwdLine->text();
	int max;
	tmp.clear();
	for(i=0;i<ordered.length();i++)
	{
		max=i;
		for(j=i+1;j<ordered.length();j++)
			if(ordered[max].toAscii()<ordered[j].toAscii())
				max=j;
		ordered.prepend(ordered[max]);
		ordered.remove(max+1,1);
	}
	int *transposition = new int[ordered.length()];
	for(i=0;i<ordered.length();i++)
		transposition[i]=pwdLine->text().indexOf(ordered[i]);
	workText.clear();
	for(i=0;i<ordered.length();i++)
		for(j=transposition[i];j<intermediate.length();j+=ordered.length())
			workText.append(intermediate[j]);
	if(finalBlanksCheck->isChecked())
	{
		z=workText.length()/finalBlanks->text().toInt();
		for(i=1;i<=z;i++)
		{
			if(!finalLineCheck->isChecked())
				workText.insert((i*(finalBlanks->text().toInt()))+i-1,QChar(' '));
			else
				workText.insert(i*(finalBlanks->text().toInt())+i-1,QChar('\n'));
		}
	}
	mw->newSubWindow(title,workText);
	close();
}

#include "ADFGVX.moc"
