/***************************************************************************
 *   Copyright (C) 2007 by Andrea Moro   *
 *   andrea_moro@email.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/
#include <QtGui>
#include "CaesarRot13.h"
#include "mdichild.h"
#include "mainWindow.h"
#include "textOpt.h"
#include "Text.h"
#include "libClassicalCrypto/libClassicalCrypto.h"

CaesarRot13::CaesarRot13(mainWindow *main,QString text, QString title)
{
	setupUi(this);
	workText=text;
	workTitle=title;
	mw=main;
	createConnections();
	initializeDialog();
}

void CaesarRot13::initializeDialog()
{
	QRegExp re("[0-9]*");
	QRegExpValidator *v = new QRegExpValidator(re,this);
	numbShiftLine->setValidator(v);
	QRegExp re2;
	if(mw->options.text_opt.keep_case)
		re2=QRegExp("["+mw->options.text_opt.alphabet+mw->options.text_opt.alphabet.toLower()+"]");
	else
		re2=QRegExp("["+mw->options.text_opt.alphabet+"]");
	QRegExpValidator *v2 = new QRegExpValidator(re2,this);
	shiftLine->setValidator(v2);
	shiftLine->setText("");
	realAlph->setText(mw->options.text_opt.alphabet);
	if(mw->options.text_opt.alphabet.length()%2!=0)
	{
		rotRadio->setEnabled(false);
		rotRadio->setChecked(false);
		caesarRadio->setChecked(true);
	}
	if(mw->options.text_opt.alphabet.length()==0)
	{
		cryptAlph->setText("");
		shiftLabel->setText("");
		numbShiftLine->setText("");
		encryptButton->setEnabled(false);
		decryptButton->setEnabled(false);
		rotRadio->setEnabled(false);
		caesarRadio->setEnabled(false);
	}
	char tmp[10];
	sprintf(tmp,"%d",mw->options.text_opt.alphabet.length());
	label_5->setText(tr("Mapping of the alphabet (")+QString(tmp)+tr(") characters)"));
	if(caesarRadio->isChecked())
		caesarSel();
	else
		rotSel();
	recalculateAlph();
}

void CaesarRot13::createConnections()
{
	connect(caesarRadio,SIGNAL(clicked()),this,SLOT(caesarSel()));
	connect(rotRadio,SIGNAL(clicked()),this,SLOT(rotSel()));
	connect(shiftLine,SIGNAL(textEdited(const QString &)),this,SLOT(recalculateAlph()));
	connect(numbShiftLine,SIGNAL(textEdited(const QString &)),this,SLOT(recalculateAlph()));
	connect(encryptButton,SIGNAL(clicked()),this,SLOT(encrypt()));
	connect(decryptButton,SIGNAL(clicked()),this,SLOT(decrypt()));
	connect(textOptButton,SIGNAL(clicked()),mw,SLOT(opt_text()));
	connect(alphRadio,SIGNAL(clicked()),this,SLOT(recalculateAlph()));
	connect(numbRadio,SIGNAL(clicked()),this,SLOT(recalculateAlph()));
	connect(mw->opttext_changed,SIGNAL(triggered()),this,SLOT(initializeDialog()));
}

void CaesarRot13::caesarSel()
{
	shiftLine->setEnabled(alphRadio->isChecked());
	numbShiftLine->setEnabled(numbRadio->isChecked());
}

void CaesarRot13::rotSel()
{
	numbShiftLine->setEnabled(false);
	shiftLine->setEnabled(false);
	int half=mw->options.text_opt.alphabet.length()/2-1;
	shiftLine->setText(mw->options.text_opt.alphabet.at(half));
	char tmp[10];
	if(mw->options.text_opt.aEqual1)
		sprintf(tmp,"%d",half+1);
	else
		sprintf(tmp,"%d",half);
	numbShiftLine->setText(QString(tmp));
	recalculateAlph();
}

void CaesarRot13::recalculateAlph()
{
	if(shiftLine->text()[0].isLower()&&mw->options.text_opt.keep_case)
		shiftLine->setText(shiftLine->text()[0].toUpper());
	if(alphRadio->isChecked())
	{
		shiftLine->setEnabled(!rotRadio->isChecked());
		numbShiftLine->setEnabled(false);
		if(shiftLine->text().length()==0)
		{
			cryptAlph->setText("");
			shiftLabel->setText("");
			numbShiftLine->setText("");
			encryptButton->setEnabled(false);
			decryptButton->setEnabled(false);
		}
		else
		{
			QString alph=mw->options.text_opt.alphabet,newAlph;
			char tmp[4];
			//j = the key
			int i,j=alph.indexOf(shiftLine->text()[0]);
			sprintf(tmp,"%d",j);
			shiftLabel->setText(tmp);
			sprintf(tmp,"%d", j+(mw->options.text_opt.aEqual1?1:0));
			numbShiftLine->setText(tmp);
			for(i=0;i<alph.length();i++)
				newAlph.append(alph[(j+i)%alph.length()]);
			cryptAlph->setText(newAlph);
			encryptButton->setEnabled(true);
			decryptButton->setEnabled(true);
		}
	}
	else
	{
		shiftLine->setEnabled(false);
		numbShiftLine->setEnabled(!rotRadio->isChecked());
		QString alph=mw->options.text_opt.alphabet;
		if((numbShiftLine->text().length()==0)||((numbShiftLine->text().toInt()<0)||
			(numbShiftLine->text().toInt()>alph.length()+(mw->options.text_opt.aEqual1?1:0))))
		{
			cryptAlph->setText("");
			shiftLabel->setText("");
			shiftLine->setText("");
			encryptButton->setEnabled(false);
			decryptButton->setEnabled(false);
		}
		else
		{
			QString newAlph;
			char tmp[10];
			int i,j=numbShiftLine->text().toInt()-(mw->options.text_opt.aEqual1?1:0);
			shiftLine->setText(QChar(alph[j]));
			sprintf(tmp,"%d",j);
			shiftLabel->setText(QString(tmp));
			for(i=0;i<alph.length();i++)
				newAlph.append(alph[(j+i)%alph.length()]);
			cryptAlph->setText(newAlph);
			encryptButton->setEnabled(true);
			decryptButton->setEnabled(true);
		}
	}
}

void CaesarRot13::decrypt()
{
	QString title;
	if(rotRadio->isChecked())
		title.append(tr("ROT 13 decryption of <"));
	else
		title.append(tr("Caesar decryption of <"));
	title.append(workTitle);
	title.append(tr(">, key <"));
	title.append(shiftLine->text());
	title.append(">");

	QString newWorkText;
	
	Qt::CaseSensitivity cs;
	if(mw->options.text_opt.keep_case)
			cs = Qt::CaseInsensitive;
		else
			cs = Qt::CaseSensitive;
	QString alphabet = mw->options.text_opt.alphabet;
	Text theText(workText, &alphabet, cs);
	int key = alphabet.indexOf(shiftLine->text().at(0));
	if (mw->options.text_opt.aEqual1) {
		//add 1 because indexes begin at 0
		//TODO : this SHALL be in the Text class !!!
		key++;
	}
	LibClassicalCrypto::CaesarEngine ciphEng(mpz_class(key), 
						 mpz_class(alphabet.length()));
	theText.setResultingText(ciphEng.decipher(theText.getIntRepr()));
	newWorkText = theText.getResultingText();
	
	mw->newSubWindow(title,newWorkText);
	close();
}

void CaesarRot13::encrypt()
{
	QString title;
	if(rotRadio->isChecked())
		title.append(tr("ROT 13 encryption of <"));
	else
		title.append(tr("Caesar encryption of <"));
	title.append(workTitle);
	title.append(tr(">, key <"));
	title.append(shiftLine->text());
	title.append(">");
	
	// the ciphertext
	QString newWorkText;
	
	Qt::CaseSensitivity cs;
	if(mw->options.text_opt.keep_case)
			cs = Qt::CaseInsensitive;
		else
			cs = Qt::CaseSensitive;
	QString alphabet = mw->options.text_opt.alphabet;
	Text theText(workText, &alphabet, cs);

	int key = alphabet.indexOf(shiftLine->text().at(0));
	if (mw->options.text_opt.aEqual1) {
		//add 1 because indexes begin at 0
		key++;
	}
	LibClassicalCrypto::CaesarEngine ciphEng(mpz_class(key), 
						 mpz_class(alphabet.size()));
	theText.setResultingText(ciphEng.encipher(theText.getIntRepr()));
	newWorkText = theText.getResultingText();
	
	mw->newSubWindow(title,newWorkText);
	close();
}

#include "CaesarRot13.moc"
