/***************************************************************************
 *   Copyright (C) 2007 by Andrea Moro   *
 *   andrea_moro@email.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef CAESARROT13_H
#define CAESARROT13_H

#include "ui_CaesarRot13.h"

class mainWindow;

/** 
 * @short User interface for Caesar and ROT 13 ciphers.
 *
 * The Caesar and ROT 13 ciphers. This class provides the user interface
 * handling and the call for encryption/decryption methods.
 */
class CaesarRot13 :
		public QWidget,
		public Ui::CaesarRot13
{
	Q_OBJECT
public:
	/**
	 * Creates a new CaesarRot13 instance.
	 *
	 * @param *main the main window of CrypTooLinux.
	 * @param text the text that will be either enciphered or deciphered.
	 * @param title the title of the window.
	 *
	 */
	CaesarRot13(mainWindow *main, QString text, QString title);

private slots:
	/**
	 * Selects the Caesar cipher and as consequence, activates the
	 * shiftLine containing the key.
	 */
	void caesarSel();
	/**
	 * Selects the ROT13 cipher and desactivates the
	 * shiftLine containing the key, set it to "M" and define the crypted
	 * alphabet.
	 */
	void rotSel();
	/**
	 * Recalculates the crypting alphabet.
	 */
	void recalculateAlph();
	/**
	 * Deciphers the message and updates all UI items (crypting alphabet,
	 * title, etc.)
	 */
	void decrypt();
	/**
	 * Ciphers the message and updates all UI items (crypting alphabet,
	 * title, etc.)
	 */
	void encrypt();
	void initializeDialog();
private:
	/**
	 * Makes the connections between signal of the ui and the slot functions.
	 */
	void createConnections();

	/**
	 * The text to encrypt or decrypt.
	 */
	QString workText;

	/**
	 * The title of window in CrypTooLinux main window that contains the text
	 * to encrypt or decrypt.
	 */
	QString workTitle;

	/**
	 * CrypTooLinux main window.
	 */
	mainWindow *mw;
};
#endif
