/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <iostream>
#include "FactorisationFermat.h"

namespace libAnalysis
{
	FactorisationFermat::FactorisationFermat(const mpz_class& n)
	{
		this->current = new mpz_class(n);
		this->upperBound = new mpz_class((n+9)/6); //cf "Primes Numbers, A Computational Perspective"
	}
	
		
	FactorisationFermat::~FactorisationFermat()
	{
		delete this->current;
		delete this->upperBound;
	}
	
	mpz_class FactorisationFermat::getNextFactor() throw(IsPrimeException)
	{
		if (*(this->current) <= 3) {
			throw IsPrimeException();
		}
		if ((*(this->current) / 2) * 2 == *(this->current)) {
			*(this->current) /= 2;
			return mpz_class(2);
		}

		mpz_class a = sqrt(*(this->current));
		if (a * a != *(this->current)) a++;
		mpz_class bSquare = a * a - *(this->current);
		//NOTE : NO NEED TO KEEP THE SQUARE ROOT BOUND !!!
		//JUST GO ON FROM THE LAST FACTOR !

		while ((a <= *(this->upperBound)) && (bSquare != sqrt(bSquare) * sqrt(bSquare))) {
			a ++;
			bSquare = a * a - *(this->current);
		}
		if ( a - sqrt(bSquare) != 1 && a != *(this->upperBound) ) {
			//NOTE : may be fastened using GMP C routines, as the
			//division is known to be exact. We maintain clarity of
			//the code at the expense of possible time.
			*(this->current) = a + sqrt(bSquare); //current is now the factor a + b
			return a - sqrt(bSquare); // returns the other factor, ie a 1 b
		}
		else {
			//no factor was found
			throw IsPrimeException();
		}
	}

	mpz_class FactorisationFermat::getRemainingQuotient()
	{
		return *(this->current);
	}

} //namespace libAnalysis
