/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <iostream>
#include "libAnalysisTest.h"

using namespace libAnalysis;

int errorCode = 0;

int main(int argc, char** argv)
{
	bool error = false;
	std::cout << "Running libAnalysis tests..."<< std::endl;

	
	
	error = error || testBruteforceFactoring(mpz_class("123456789456", 10));
	error = error || testFermatFactoring(mpz_class("1234567890", 10));
	error = error || testCustomFactoring(mpz_class("1234567890", 10));
	
	std::cout << "Done." << std::endl;
	if (error) {
		
		return errorCode;
	}
	else {
		std::cout << "All tests are ok." << std::endl;
		return 0;
	}
	
}

int testBruteforceFactoring(const mpz_class& n)
{
	errorCode++;
	std::cout << "Testing the bruteforce factoring algorithm." << std::endl;
	//we don't follow the recommendation of the doc in order to test !
	//but it is dirty : using an exception as the only way of getting out an
	//infinite loop is strange.
	try {
		FactorisationBruteforce engine = FactorisationBruteforce(n);
		while (true) {
			std::cout << engine.getNextFactor() << " " << std::endl;
		}
	} catch (const IsPrimeException& e) {
		std::cout << "The number " << n << " is fully factorised." << std::endl;
	}
	
	std::cout << "Done (bruteforce factoring)." << std::endl << std::endl;
	return 0;
}

int testFermatFactoring(const mpz_class& n)
{
	errorCode++;
	std::cout << "Testing the Fermat factoring algorithm." << std::endl;
	//we don't follow the recommendation of the doc in order to test !
	//but it is dirty : using an exception as the only way of getting out an
	//infinite loop is strange.
	try {
		FactorisationFermat engine = FactorisationFermat(n);
		while (true) {
			std::cout << "Found factor: " << engine.getNextFactor() << " " << std::endl;
			std::cout << "Remaining quotient : " << engine.getRemainingQuotient() << " " << std::endl;
		}
	} catch (const IsPrimeException& e) {
		std::cout << "The number " << n << " is partially factorised." << std::endl;
	}
	
	std::cout << "Done (Fermat factoring)." << std::endl << std::endl;
	return 0;
}

int testCustomFactoring(const mpz_class& n)
{
	errorCode++;
	std::cout << "Testing the custom factoring algorithm." << std::endl;
	
	CustomFactorisation engine = CustomFactorisation(n);
	while (engine.isPrime() == 0) {
                mpz_class factor = engine.getNextFactor();
		std::cout << "Found factor: " << factor << " " << std::endl;
	}
	std::cout << "Last factor: " << engine.getRemainingQuotient() << std::endl;
	
	std::cout << "Done (custom factoring)." << std::endl << std::endl ;
	return 0;
}
