/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <iostream>
#include "libClassicalAnalysisTest.h"

using namespace libClassicalAnalysis;

int errorCode = 0;

int main(int argc, char** argv)
{
	std::cout << "Running libClassicalAnalysis tests..." << std::endl;
	bool error = false;
	

	std::vector<mpz_class> text;
	std::vector<mpz_class> alphabet;
	initialiseText(text, alphabet);

	
	error = error || testFrequencies(text, alphabet);
	
	
	std::cout << "Done. " << std::endl;
	if (error) {
		return errorCode;
	}
	else {
		std::cout << "All tests were ok." << std::endl;
		return 0;
	}
	
		
}


int testFrequencies(const std::vector<mpz_class>& text, const std::vector<mpz_class>& alphabet)
{
	errorCode++;
	std::cout << "Testing the frequencies computation..." << std::endl;
	
	std::cout << "The text : " << std::endl;
	for (std::vector<mpz_class>::size_type i = 0 ; i < text.size() ; i++) {
		std::cout << text.at(i) << " ";
	}
	
	std::cout << "Frequencies : " << std::endl;
	
	Frequencies engine = Frequencies(text, alphabet);
	std::vector<mpz_class> singleFreq = engine.getSingleFrequencies();
	
	for (std::vector<mpz_class>::size_type i = 0 ; i < alphabet.size() ; i++) {
		std::cout << "Letter : " << alphabet.at(i) << " . Frequency : " << singleFreq.at(i) << std::endl;
	}
	
	std::vector<std::vector<mpz_class> > twoFreq = engine.getTwoFrequencies();
	std::vector<std::vector<mpz_class> >::size_type j,k;


	//pretty printing
	std::cout << "Letters  ";
	for (j = 0 ; j < alphabet.size() ; j++) {
		std::cout << alphabet.at(j) << " ";
		if (alphabet.at(j) <= 9) {
			std::cout << " ";
		}
	}
	std::cout << std::endl;
	for (j = 0 ; j < twoFreq.size() ; j++) {
		std::cout << alphabet.at(j) << "       ";
		if (alphabet.at(j) <= 9) {
			std::cout << " ";
		}
		for (k = 0 ; k < twoFreq.at(j).size() ; k++) {
			std::cout << twoFreq.at(j).at(k) << " ";
			if (twoFreq.at(j).at(k) <= 9) {
				std::cout << " ";
			}
		}
		std::cout << std::endl;
	}
	
	

	std::cout << "Done (frequencies)." << std::endl << std::endl;
	return 0;
}

void initialiseText(std::vector<mpz_class>& text, std::vector<mpz_class>& alphabet)
{
	std::vector<mpz_class>::size_type i;
	for (i = 0 ; i < 26 ; i++) {
		alphabet.push_back(mpz_class(i));
	}
	
	
	mpz_class rnd = 5;
	for (i = 0 ; i < 351 ; i++) {
		text.push_back(rnd);
		
		//stupid linear random generator
		rnd = (rnd * mpz_class(3) + 11) % mpz_class(26);
	}
	
}
