/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __LIB_ALGEBRA_NON_INVERTIBLE_ELEMENT_EXCEPTION_H
#define __LIB_ALGEBRA_NON_INVERTIBLE_ELEMENT_EXCEPTION_H

#include <exception>

namespace libalgebra
{
	/**
	 * Exception thrown when a division by a non invertible element is
	 * tried in a Ring Element. It contains a copy of the element, of type T.
	 *
	 * @author apterium
	 */
	template<class T>
	class NonInvertibleElementException : public std::exception
	{
	public:
		/**
		 * Constructs a new NonInvertibleElementException instance,
		 * given the non invertible element in question.
		 *
		 * @param element a constant reference to the element.
		 */
		NonInvertibleElementException(const T& element);
		
		/**
		 * Copy constructor.
		 * 
		 * 
		 */
		NonInvertibleElementException(const NonInvertibleElementException<T>& copy);
		
		/**
		 * The destructor of the exception.
		 * 
		 */
		~NonInvertibleElementException() throw();
	private:
		T* theElement;
	};
	
}

#include "NonInvertibleElementException.tpp"

#endif //__LIB_ALGEBRA_NON_INVERTIBLE_ELEMENT_EXCEPTION_H
