/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __LIB_ALGEBRA_NON_PRIME_EXCEPTION_H
#define __LIB_ALGEBRA_NON_PRIME_EXCEPTION_H

#include <gmpxx.h>
#include <exception>


namespace libalgebra
{
	/**
	 * Exception thrown when attempting to construct a new Fp field with p non
	 * prime.
	 *
	 * @author apterium
	 */
	class NonPrimeException : public std::exception
	{
	public:
		/**
		 * Constructs a new NonPrimeException instance,
		 *
		 * @param number the non prime number
		 */
		NonPrimeException(const mpz_class &number);
		
		/**
		 * The destructor of the exception.
		 * 
		 */
		~NonPrimeException() throw();
		
		/**
		 * Gets the non prime number which was the occasion to raise this
		 * exception.
		 *
		 * @return the non prime number.
		 */
		mpz_class& getTheNumber() const;
		
	private:
		mpz_class* nmbr;
	};
	
	

}

#endif //__LIB_ALGEBRA_NON_PRIME_EXCEPTION_H
