/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __LIB_ALGEBRA_RING_H
#define __LIB_ALGEBRA_RING_H

#include <gmpxx.h>
#include "Field.h"
#include "NonIntegralDomainException.h"


namespace libalgebra
{
  /**
   * Basic prototype for a (non commutative) Ring. 
   *
   * @author apterium
   */
  class Ring : virtual public Field
  {
    
  public:
	  
	  virtual ~Ring()
	  {
	  };
	  
	  /**
	   * Tells whether or not this Ring is an integral domain. That is,
	   * for all a, b elements of this Ring, a.b (a times b)  = 0 if and
	   * only if a = 0 or b = 0.
	   * Note that the Rings which will be implemented shall provide
	   * themselve a proper and well-documented mechanism to tell the
	   * result. For example, the documentation shall precise if the
	   * method used to determine the answer is mathematically true or
	   * if there is only some probability it is (for example: Z/pZ is integral if
	   * and only if p is prime. The primality testing of p may be only
	   * a compositeness test, like Miller-Rabin test, so non prime
	   * numbers may pass the test) or, worse, if it answers always
	   * false due to a lack of implementation.
     *
     *
     * @return true if this Ring is an integral domain with some
     * probability, according to the documentation.
     */
    virtual bool isIntegralDomain() const = 0;
    
    /**
     * Tells whether this Ring is euclidean or not. 
     * 
     * @return true if and only if this Ring is Euclidean.
     */
    virtual bool isEuclidean() const = 0;
    
  };
	
}

#endif //__LIB_ALGEBRA_RING_H
