/***************************************************************************
 *   Copyright (C) 2007 by Andrea Moro   *
 *   andrea_moro@email.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef MDICHILD_H
#define MDICHILD_H

#include <QTextEdit>

/**
 * @short This class manage the MDI interface system.
 *
 * This class permits the edition a text field, and provides basic
 * functionalities like loading/saving and detection of edition.
 *
 */
class MdiChild :
		public QTextEdit
{
     Q_OBJECT

public:
       /**
        * Constructor. Creates a new MdiChild instance.
        */
	MdiChild();
       /**
        * Creates a new, empty file and associates it with this midchild.
	*/
	void newFile();
	/**
	 * Loads a file.
	 * 
	 * @param fileName the file name.
	 * @return true if the file is read correctly, else false
	 */
	bool loadFile(const QString &fileName);
	/**
	 * Saves the current file.
	 *
	 * @return true if the file is saved correctly, else false
	 */
	bool save();
	/**
	 * Saves the current file under a new name, thus opens a new
	 * file dialog requesting for the name.
	 *
	 * @return true if the file is saved correctly, else false
	 */
	bool saveAs();
	/**
	 * Saves the current file under the provided name.
	 *
	 * @param fileName a QString representing the file name.
	 * @return true if the file is saved correctly, else false
	 */
	bool saveFile(const QString &fileName);
	/**
	 * @return Returns the file name without the full directory tree.
	 */
	QString userFriendlyCurrentFile();
	/**
	 * @return Returns the current file name. If no one was defined (when
	 * a new file creation is requested for example, when the user
	 * doesn't have saved it yet), a default one is assigned.
	 */
	QString currentFile() { return curFile; }
protected:
	/**
	 * Handle a QCloseEvent. Basically, if the file was modified
	 * but not saved, a confirmation pop up will appear.
	 *
	 * @param *event a QCloseEvent.
	 */
	void closeEvent(QCloseEvent *event);
private slots:
	/**
	 * Set the window like it is modified.
	 */
	void documentWasModified();
private:
	/**
	 * If the window is modified ask to save it.
	 *
	 * @return true if the window is saved(or it is not modified), else false
	 */
	bool maybeSave();
	/**
	 * Set current file of the window.
	 */
	void setCurrentFile(const QString &fileName);
	/**
	 * Strip a path.
	 *
	 * @param &fullFileName a path
	 * @return a stripped path(only the file name)
	 */
	QString strippedName(const QString &fullFileName);
	/**
	 * The current file.
	 */
	QString curFile;
	/**
	 * True if the window is untitled, else false.
	 */
	bool isUntitled;
};

#endif
